unit UnidadMenuPrincipal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, Buttons, StdCtrls, ExtCtrls,
  ComCtrls, shellapi, inifiles;

type TformMenuPrincipal = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    txtIP: TEdit;
    txtMensaje: TEdit;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    txtSegundos: TEdit;
    Label4: TLabel;
    btEnviar: TBitBtn;
    bDetener: TBitBtn;
    bSalir: TBitBtn;
    LWEB: TLabel;
    opReiniciar: TRadioButton;
    opApagar: TRadioButton;
    opForzar: TCheckBox;
    Memo1: TMemo;
    procedure BtnQuitClick(Sender: TObject);
    procedure txtSegundosKeyPress(Sender: TObject; var Key: Char);
    procedure btEnviarClick(Sender: TObject);
    procedure bDetenerClick(Sender: TObject);
    procedure bSalirClick(Sender: TObject);
    procedure LWEBClick(Sender: TObject);
    procedure opReiniciarClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
end;

var
  formMenuPrincipal: TformMenuPrincipal;

implementation

{$R *.dfm}

const
  SE_SHUTDOWN_NAME = 'SeShutdownPrivilege';

var
  hToken: THandle;

//Lee un booleano de un INI
function leBoolINI (clave, cadena : string; defecto : boolean) : boolean;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readbool (clave, cadena, defecto);
  finally
    free;
  end;
end;

//Lee una cadena de texto de un INI
function leCadINI (clave, cadena : string; defecto : string) : string;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readString (clave, cadena, defecto);
  finally
    free;
  end;
end;


//escribe un Booleano en un INI
procedure esBoolINI (clave, cadena : string; valor : boolean);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    writeBool (clave, cadena, valor);
  finally
    free;
  end;
end;

//escribe una cadena de texto en un INI
procedure esCadINI (clave, cadena, valor : string);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    writeString (clave, cadena, valor);
  finally
    free;
  end;
end;


//Ajustar los privilegios de la mquina
procedure ajustarPrivilegios (equipo : string);
var
  hProc : THandle;
  TP : TTokenPrivileges;
  PrevToken : TTokenPrivileges;
  PSsize : DWORD;
begin
  hProc := GetCurrentProcess;
  OpenProcessToken (hProc, TOKEN_ADJUST_PRIVILEGES +
      TOKEN_QUERY, hToken);
  TP.PrivilegeCount := 1;
  LookupPrivilegeValue (PChar(equipo),
      SE_SHUTDOWN_NAME, TP.Privileges[0].Luid);
  TP.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
  AdjustTokenPrivileges (hToken, False, TP,
      SizeOf(TP), PrevToken, PSsize);
end;

//Enviar comando apagado/reinicio
function enviarComandoApagado (equipo : string; mensaje : string;
    forzar : Boolean; reiniciar : Boolean; tiempoSegundos : Integer) : boolean;
begin
  try
    ajustarPrivilegios (equipo);
  finally
    Result := InitiateSystemShutdown (PChar(equipo), PChar(mensaje),
        tiempoSegundos, forzar, reiniciar);
    CloseHandle( hToken );
  end;
end;

//Detener la orden de apagado/reiniciado
function anularApagado (equipo : string) : boolean;
begin
  try
    ajustarPrivilegios (equipo);
  finally
    result := AbortSystemShutdown (PChar(equipo));
    CloseHandle (hToken);
  end;
end;



procedure TformMenuPrincipal.BtnQuitClick(Sender: TObject);
begin
  close;
end;

procedure TformMenuPrincipal.txtSegundosKeyPress(
    Sender: TObject; var Key: Char);
begin
  if not (Key in [#8, #13, '0'..'9']) then
    Key := #0;
end;

procedure TformMenuPrincipal.btEnviarClick(Sender: TObject);
var
  resultado : boolean;
begin
  resultado := enviarComandoApagado (txtIP.Text, txtMensaje.Text, opForzar.Checked,
      opReiniciar.Checked, StrToInt(txtSegundos.Text));
  if resultado then
    MessageDlg('Comando de apagado/reiniciado enviado correctamente.',
        mtInformation, [mbok], 0)
  else
    MessageDlg('Ha habido algn error al enviar el comando de ' +
        'apagado/reiniciado. Compruebe la IP y asegrese de que tiene ' +
        'permisos suficientes para ejecutarlo.',
        mtError, [mbok], 0)
end;

procedure TformMenuPrincipal.bDetenerClick(Sender: TObject);
begin
  if anularApagado (txtIP.Text) then
    MessageDlg ('Comando de anulacin de apagado/reinicio ' +
        'enviado correctamente.',
        mtInformation, [mbok], 0)
  else
    MessageDlg ('Ha habido algn error al enviar el comando ' +
        'de anulacin de apagado/reinicio.',
        mtError, [mbok], 0);
end;

procedure TformMenuPrincipal.bSalirClick(Sender: TObject);
begin
  close;
end;

procedure TformMenuPrincipal.LWEBClick(Sender: TObject);
begin
  ShellExecute(Handle, Nil, PChar('http://www.ajpdsoft.com'),
      Nil, Nil, SW_SHOWNORMAL);
end;

procedure TformMenuPrincipal.opReiniciarClick(Sender: TObject);
begin
  if opReiniciar.Checked then
    txtMensaje.Text := 'Su equipo se va a reiniciar en ' +
        txtSegundos.Text +
        ' segundos!! Si quiere anular el reinicio utilice el' +
        ' comando "shutdown"' +
        ' con el parmetro "a" en el equipo remoto.'
  else
    txtMensaje.Text := 'Su equipo se va a apagar en ' +
        txtSegundos.Text +
        ' segundos!! Si quiere anular el apagado utilice el' +
        ' comando "shutdown"' +
        ' con el parmetro "a" en el equipo remoto.'
end;

procedure TformMenuPrincipal.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  esCadINI('Configuracin', 'IP', txtIP.Text);
  esCadINI('Configuracin', 'Mensaje', txtMensaje.Text);
  esCadINI('Configuracin', 'Retrasar segundos', txtSegundos.Text);
  esBoolINI('Configuracin', 'Forzar', opForzar.Checked);
  esBoolINI('Configuracin', 'Reiniciar', opReiniciar.Checked);
  esBoolINI('Configuracin', 'Apagar', opApagar.Checked);
end;

procedure TformMenuPrincipal.FormCreate(Sender: TObject);
begin
  txtIP.Text := leCadINI('Configuracin', 'IP', '');
  txtSegundos.Text := leCadINI('Configuracin',
      'Retrasar segundos', '30');
  opForzar.Checked := leBoolINI('Configuracin', 'Forzar', true);
  opReiniciar.Checked := leBoolINI('Configuracin',
      'Reiniciar', true);
  opApagar.Checked := leBoolINI('Configuracin', 'Apagar', false);
  txtMensaje.Text := leCadINI('Configuracin', 'Mensaje', '');
end;

end.
